@@ -1,5 +1,6 @@
 from io import BytesIO
 from io import StringIO
+from typing import Any
 from typing import Callable
 
 from actionpack import Action
@@ -14,8 +15,12 @@ class FakeAction(Action[Name, Outcome]):
     def __init__(
         self,
         name: Name = None,
-        instruction_provider: Callable = None,
+        typecheck: Any = None,
+        instruction_provider: Callable[[], Any] = None,
     ):
+        if typecheck:
+            raise TypeError(str(typecheck))
+
         self.name = name
         self.instruction_provider = instruction_provider
         self.state = {'this': 'state'}
